@@ -71,6 +71,10 @@ def generate_mappings():
                            0)
                    == float(groups[1]))
 
+    @add_partial('(\S*) starts with (\S*)\?')
+    def x_startswith_y(activity, groups):
+        return groups[0].startswith(groups[1])
+
     @add_partial('(\S*) exists (\S*) times?\?')
     def exist_times(activity, groups):
         return len(rm_blank(activity.xpath(groups[0]))) == int(groups[1])
@@ -106,18 +110,35 @@ def generate_mappings():
             return default
         return bool(int(rm_blank(activity.xpath(xpath)[0])) >= int(amount))
 
+    def exist_check_two_list(activity, xpath_one, xpath_two, codelist):
+        outcome = False
+        data = activity.xpath(xpath_one)+activity.xpath(xpath_two)
+        for d in data:
+            try:
+                if bool(str(d) in codelist):
+                    outcome = True  
+                elif bool(str(d).lowercase in codelist):
+                    outcome = True 
+                elif bool(str(d).uppercase in codelist):
+                    outcome = True
+                return outcome
+            except Exception:
+                return False
+
     def exist_check_list(activity, xpath, codelist):
         outcome = False
-        try:
-            if bool(str(activity.xpath(xpath)[0]) in codelist):
-                outcome = True  
-            elif bool(str(activity.xpath(xpath)[0]).lowercase in codelist):
-                outcome = True 
-            elif bool(str(activity.xpath(xpath)[0]).uppercase in codelist):
-                outcome = True
-            return outcome
-        except Exception:
-            return False
+        data = activity.xpath(xpath)
+        for d in data:
+            try:
+                if bool(str(d) in codelist):
+                    outcome = True  
+                elif bool(str(d).lowercase in codelist):
+                    outcome = True 
+                elif bool(str(d).uppercase in codelist):
+                    outcome = True
+                return outcome
+            except Exception:
+                return False
 
     def x_months_ago_check(activity, xpath, months, many=False):
         outcome = False
@@ -158,6 +179,14 @@ def generate_mappings():
     def exist_list(data, groups):
         return exist_check_list(data['activity'], groups[0], data['lists'][groups[1]])
 
+    @add_partial_with_list('at least one of (\S*) is on list (\S*)\?') 
+    def atleastone_x_on_list_z(data, groups):
+        return exist_check_list(data['activity'], groups[0], data['lists'][groups[1]])
+
+    @add_partial_with_list('at least one of \((\S*) or (\S*)\) is on list (\S*)\?')
+    def atleastone_x_y_on_list_z(data, groups):
+        return exist_check_two_list(data['activity'], groups[0], groups[1], data['lists'][groups[2]])
+
     @add_partial('(\S*) or (\S*) \(for each (\S*)\) is less than (\S*) months? ago\?')
     def less_than_x_months_ago(activity, groups):
         return (x_months_ago_check(activity, groups[0], groups[3]) or 
@@ -180,14 +209,14 @@ def generate_mappings():
 
     ## Conditional tests (only run if something)
     
-    @add_partial('(\S*) exists \(if (\S*) is at least (\S*)\)\?') 
+    @add_partial('(\S*) exists\? \(if (\S*) is at least (\S*)\)') 
     def exist_if_gte(activity, groups):
         if check_value_gte(activity, groups[1], groups[2], True):
             return exist_check(activity, groups[0])
         else:
             return None
 
-    @add_partial('(\S*) or (\S*) exists \(if (\S*) is at least (\S*)\)\?') 
+    @add_partial('(\S*) or (\S*) exists\? \(if (\S*) is at least (\S*)\)') 
     def exist_or_if_gte(activity, groups):
         if check_value_gte(activity, groups[2], groups[3], True):
             return (exist_check(activity, groups[0]) or 
@@ -195,14 +224,14 @@ def generate_mappings():
         else:
             return None
 
-    @add_partial('(\S*) exists \(if (\S*) is at least (\S*) and (\S*) is not (\S*)\)\?') 
+    @add_partial('(\S*) exists\? \(if (\S*) is at least (\S*) and (\S*) is not (\S*)\)') 
     def exist_if_both(activity, groups):
         if (check_value_gte(activity, groups[1], groups[2], True) and not (check_value_is(activity, groups[3], groups[4], False))):
             return exist_check(activity, groups[0])
         else:
             return None
 
-    @add_partial('(\S*) exists \(if (\S*) is at least (\S*) and \((\S*) or (\S*) is not (\S*)\)\)\?') 
+    @add_partial('(\S*) exists\? \(if (\S*) is at least (\S*) and \((\S*) or (\S*) is not (\S*)\)\)') 
     def exist_if_both_or(activity, groups):
         if (check_value_gte(activity, groups[1], groups[2], True) and not (check_value_is(activity, groups[3], groups[5], False) or check_value_is(activity, groups[4], groups[5], False))):
             return exist_check(activity, groups[0])
@@ -210,7 +239,7 @@ def generate_mappings():
             return None
 
 
-    @add_partial('(\S*) or (\S*) exists \(if (\S*) is at least (\S*) and \((\S*) or (\S*) is not (\S*)\)\)\?') 
+    @add_partial('(\S*) or (\S*) exists\? \(if (\S*) is at least (\S*) and \((\S*) or (\S*) is not (\S*)\)\)') 
     def exist_or_if_both(activity, groups):
         if (check_value_gte(activity, groups[2], groups[3], True) and not (check_value_is(activity, groups[4], groups[5], False) or check_value_is(activity, groups[4], groups[5], False))):
             return (exist_check(activity, groups[0]) or 
